#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-rgma-gin-config v. 5.1.2
#
# Post-installation script for configuring the gLite R-GMA gadget-in
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: glite-rgma-gin-config.py,v 1.15.2.9 2006/05/02 10:27:02 flammer Exp $
#
# Usage: python glite-rgma-gin-config [-c|-v|-h|--help]
#        -c                  print configuration
#        -v                  print version
#        -h,--help           print usage info
#        --verbose           turn verbosity on
#
#        --siteconfig=URL    specify URL of the site configuration file
#        --start             start service
#        --stop              stop service
#        --configure         configure service
#        --status            print status of service (running/not running)
#        --cemon=yes|no      Run the CE backend for R-GMA Gin
#        --subservice        run script as a subservice
#
# Return codes: 0 - Ok
#               1 - Failed
#
#####################################################################################

import os,string,socket,pwd
import sys
import getopt
import commands

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams

# Set global variables here 
global params                         # all config values from the XML file

class glite_rgma_gin:

    #################################################################
    # init 
    #
    # init
    #
    #################################################################        
    def __init__(self):
        self.verbose = 0
        self.version = "5.1.2"
        self.name = "glite-rgma-gin"
        self.friendly_name = "gLite R-GMA gin"
        

    #################################################################
    # banner 
    #
    # print banner
    #
    #################################################################    
    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "            Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #################################################################
    # copyright 
    #
    # print copyright
    #
    #################################################################    
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # version
    #
    # print version
    #
    #################################################################    
    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################    
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()

        print "Usage: \n"
        print "Edit the configuration file %s.cfg.xml in " %  self.name,
        print "%s/etc.config/templates" % os.environ['GLITE_LOCATION']
        print "save it as %s/etc/config/%s.cfg.xml" % (os.environ['GLITE_LOCATION'], self.name),
        print "and run the script as follows"
        print "python %s-config [OPTION...]" % self.name
        print ''
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --verbose           turn verbosity on'
        print ''
        print '    --start             start service'
        print '    --stop              stop service'
        print '    --configure         configure service'
        print '    --status            print status of service (running/not running)'
        print ''
        print '    --cemon=yes|no      run the CE backend for R-GMA Gin'
        print '    --siteconfig=URL    URL of the site configuration file'
        print '    --subservice        run as subservice script'
        print ''
        print '\n'    


    #################################################################
    # configure
    #
    # configure rgma gin
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):
    
        # ------------------------------------------------------------
        # get corresponding configuration values
        # ------------------------------------------------------------
        
        errorFlag = 0
        
        print "Checking configuration values                             ",        
        
        # subservice                    
        if params.has_key('rgma.gin.run_as_subservice'):
            subservice = params['rgma.gin.run_as_subservice']
        else:
            subservice = "false"
                                        
        # rgma server hostname (user)
        if params.has_key('rgma.server.hostname'):
            server_hostname = params['rgma.server.hostname']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.hostname\" not found.")
            return 1                
        
        # rgma schema hostname (user)
        if params.has_key('rgma.schema.hostname'):
            schema_hostname = params['rgma.schema.hostname']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.schema.hostname\" not found.")
            return 1

        # rgma registry host: at least one must be given (user)
        if not len(params['rgma.registry.hostname']) > 0:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.registry.hostname\" not found.")
            return 1

        # run generic info provider (gip) (user)
        if params.has_key('rgma.gin.run_generic_info_provider'):
            run_generic_info_provider = params['rgma.gin.run_generic_info_provider']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.gin.run_generic_info_provider\" not found.")
            return 1
            
        # run fmon provider (user)
        if params.has_key('rgma.gin.run_fmon_provider'):
            run_fmon_provider = params['rgma.gin.run_fmon_provider']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.gin.run_fmon_provider\" not found.")
            return 1
                                
        # host certificate file (advanced)        
        if params.has_key('host.certificate.file'):
            host_certificate_file = params['host.certificate.file']
            
        # host.key.file (advanced)
        if params.has_key('host.key.file'):
            host_key_file = params['host.key.file']
            
        # ca.certificates.dir (advanced)
        if params.has_key('ca.certificates.dir'):
            ca_certificates_dir = params['ca.certificates.dir']
            
        # user certificates path (advanced)
        if params.has_key('user.certificate.path'):
            user_certificates_path = params['user.certificate.path']
            
        # rgma user name (advanced)
        if params.has_key('rgma.user.name'):
            rgma_user_name = params['rgma.user.name']
            
        # rgma user group (advanced)
        if params.has_key('rgma.user.group'):
            rgma_user_group = params['rgma.user.group']        
            
        # run ce provider => default=no
        if params.has_key('rgma.gin.run_ce_provider'):
            run_ce_provider = params['rgma.gin.run_ce_provider']
        else:
            run_ce_provider = "no"
        
        # check providers consistency
        if run_ce_provider == "yes" and run_generic_info_provider == "yes":
            glib.printFailedMessage()
            glib.printErrorMessage("The R-GMA CE Provider and the Generic Info Provider (GIP) cannot be used together")
            glib.printErrorMessage("Please set either rgma.gin.run_ce_provider or rgma.gin.run_generic_info_provider to 'no'")
            glib.printErrorMessage("If GIn is installed as part of a gLite CE service, it is recommended to disable GIP")
            return 1
            
        # rgma secure mode
        if params.has_key('rgma.secure.mode'):
            rgma_secure_mode = params['rgma.secure.mode']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.secure.mode\" not found.")
            return 1    
            
        glib.printOkMessage()                    

        # temporary message as long as only one registry is allowed
        if len(params['rgma.registry.hostname']) > 1:
            print
            glib.printWarningMessage("You specified more than one registry.")
            glib.printWarningMessage("At the moment only one registry is allowed.")
            glib.printWarningMessage("Configuration will use first specified registry '%s'" % params['rgma.registry.hostname'][0])
            glib.printWarningMessage("and ignore the others.")
            print
            
        # ------------------------------------------------------------        
        # Check certificates (if enabled and not run as subservice)
        # ------------------------------------------------------------    
        if subservice=="false":
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if self.verbose:
                        print "Checking certificates                                          "
                    else:
                        print "Checking certificates                                          ",                        
                    if glib.check_certs(params) != 0:
                        if self.verbose:
                            print "Checking certificates                                          ",                                                
                        glib.printFailedMessage()
                        return 1
                    else:
                        if self.verbose:
                            print "Checking certificates                                          ",                                                    
                        glib.printOkMessage()
                                        
        # ------------------------------------------------------------
        # Run security util script (if not run as subservice)
        # ------------------------------------------------------------
                
        if subservice=="false":
            print "Configuring security ...                                 "
            errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])

            if errorcode:
                print "Configuring security                                      ",
                glib.printFailedMessage()
                return 1
        
            print "Configuring security                                      ",
            glib.printOkMessage()
            
        #-------------------------------------------------------------------------------
        # Verify the $GLITE_LOCATION/etc directory structure
        #-------------------------------------------------------------------------------
        
        print "Verifying the %s/etc directory structure          " % os.environ['GLITE_LOCATION'],
    
        path = '%s/etc' % os.environ['GLITE_LOCATION']
        if not os.path.exists(path):
            glib.printFailedMessage()
            glib.printErrorMessage('%s/etc does not exist! Please verify your gLite installation' % os.environ['GLITE_LOCATION'])
            return 1                        
        
        glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Verify the JAVA installation and set it in /etc/java.conf
        #-------------------------------------------------------------------------------

        if glib.checkJava():
            return 1    
        
        # ------------------------------------------------------------
        # Create R-GMA group and user
        # ------------------------------------------------------------

        if self.verbose == 0:
            print "Creating R-GMA user and group                             ",
        else:
            print "Creating R-GMA user and group ...                         "
        
        if glib.add_user(rgma_user_name,rgma_user_group) == -1:
            glib.printFailedMessage()
            return 1

        if self.verbose == 0:            
            glib.printOkMessage()
        else:
            print "Creating R-GMA user and group                             ",
            glib.printOkMessage()
    
        # ------------------------------------------------------------
        # Configure R-GMA
        # ------------------------------------------------------------

        registries_string = ""

# the following line should be put back as soon as multiple registries are enabled again            
####
#        for item in params['rgma.registry.hostname']:
#
#            registries_string = registries_string + " --registry %s" % item                        
#            if self.verbose:
#                print "Setting up R-GMA to use registry server '%s'" % item
####

        # the following lines should be removed as soon as multiple registries are enabled again
        # start-new-codeblock
        registries_string = "--registry %s" % params['rgma.registry.hostname'][0]
        if self.verbose:
            print "Setting up R-GMA to use registry server '%s'" % params['rgma.registry.hostname'][0]
        # end-new-codeblock
    
        print "Configuring R-GMA                                         ",        

        errorcode =0
        errorcode = glib.export('RGMA_HOME', os.environ['GLITE_LOCATION'])

        if errorcode:
            glib.printFailedMessage()
            glib.printErrorMessage("Could not export RGMA_HOME environment variable.")
            return 1
    
        # decide on security setting
        if rgma_secure_mode=="true":
            secure_setting="--secure=yes"
        else:
            secure_setting="--secure=no"
            
        # add server, schema and registry(ies)                                    
        errorcode = 0
        if self.verbose == 0:
            errorcode += os.system('python %s/share/rgma/scripts/rgma-setup.py --server %s --schema %s %s %s --silent'  \
                % (os.environ['GLITE_LOCATION'], server_hostname, schema_hostname, registries_string, secure_setting))
        else:
            errorcode += os.system('python %s/share/rgma/scripts/rgma-setup.py --server %s --schema %s %s %s'  \
                % (os.environ['GLITE_LOCATION'], server_hostname, schema_hostname, registries_string, secure_setting))
        
        if errorcode:
            glib.printFailedMessage()
            return 1

        glib.printOkMessage()            
    
        # ------------------------------------------------------------
        # Updating host key and certificate for R-GMA
        # ------------------------------------------------------------

        print "Updating hostkeys and certificates for R-GMA              ",

        rgma_user_home_dir = pwd.getpwnam(rgma_user_name)[5]

        user_cert_filename = "hostcert.pem"
        user_key_filename  = "hostkey.pem"

        # create dir user_certificates_path if it does not exist
        if not os.path.exists("%s/%s" % (rgma_user_home_dir, user_certificates_path)):
            os.mkdir("%s/%s" % (rgma_user_home_dir, user_certificates_path),0755)

        # copy key and cert file        
        errorcode = 0
        errorcode += os.system("cp %s %s/%s/%s" % \
            (host_key_file, rgma_user_home_dir, user_certificates_path, user_key_filename))/256
        errorcode += os.system("cp %s %s/%s/%s" % \
            (host_certificate_file, rgma_user_home_dir, user_certificates_path, user_cert_filename))/256
        if errorcode:
            glib.printFailedMessage()
            glib.printErrorMessage("Could not copy certificate and key for user '%s'" % rgma_flexible_archiver_user_id)
            glib.printErrorMessage("to files '%s/%s/%s' and '%s/%s/%s'" % \
                (rgma_user_home_dir, user_certificates_path, user_cert_filename, \
                  rgma_user_home_dir, user_certificates_path, user_key_filename))
            return 1
        
        # change owner of key and cert file        
        errorcode = 0
        errorcode += os.system("chown %s:%s %s/%s/%s" % \
            (rgma_user_name, rgma_user_group, \
             rgma_user_home_dir, user_certificates_path, user_key_filename))/256
        errorcode += os.system("chown %s:%s %s/%s/%s" % \
            (rgma_user_name, rgma_user_group, \
             rgma_user_home_dir, user_certificates_path, user_cert_filename))/256
        
        if errorcode:
            glib.printFailedMessage()
            glib.printErrorMessage("Could not change access rights of certificate and key for user '%s:%s'" % \
                (rgma_user_name, rgma_user_group))
            glib.printErrorMessage("for file '%s/%s/hostkey.pem' and '%s/%s/hostcert.pem'" % \
                (rgma_user_home_dir, user_certificates_path))
            return 1

        glib.printOkMessage()

        # ------------------------------------------------------------
        # configure R-GMA gin security
        # ------------------------------------------------------------        
        print "Configuring %s security                      " % service.friendly_name,
        
        path = "%s/etc/rgma/ClientAuthentication.props" % os.environ['GLITE_LOCATION']

        try:
            file = open(path, 'w')
            file.write('sslCertFile=%s/%s/%s\n' % (rgma_user_home_dir, user_certificates_path, user_cert_filename))
            file.write('sslKey=%s/%s/%s\n' % (rgma_user_home_dir, user_certificates_path, user_key_filename))
            file.write('crlFiles=%s/*.r0\n' % ca_certificates_dir)
            file.write('sslCAFiles=%s/*.0\n' % ca_certificates_dir)
            file.write('#gridProxyFile=/tmp/grid-user-proxy.pem\n')
            file.close()
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                glib.printFailedMessage()
                glib.printErrorMessage("File '%s' is not writable" % path)
                return 1
            else:
                glib.printFailedMessage()
                raise ex
                return 1

        glib.printOkMessage()    
                
        # ------------------------------------------------------------
        # configure R-GMA gin
        # ------------------------------------------------------------        
        
        print "Configuring %s                               " % service.friendly_name,                
        
        errorcode = os.system("%s/bin/rgma-gin-config --gip=%s --fmon=%s --glite-ce=%s" % \
            (os.environ['GLITE_LOCATION'], run_generic_info_provider, run_fmon_provider, run_ce_provider))
        if errorcode:
            glib.printFailedMessage()
            glib.printErrorMessage("Running %s configuration script '%s/bin/rgma-gin-config --gip=%s --fmon=%s --glite-ce=%s' failed." % \
                (service.friendly_name, os.environ['GLITE_LOCATION'], run_generic_info_provider, run_fmon_provider, run_ce_provider))                            
            return 1
            
        glib.printOkMessage()
        
        # add to gLite service control
        if subservice=="false":
            glib.registerService()
            
        return 0
        
            
    #################################################################
    # start
    #
    # start gin
    #
    # return 0 ok
    #        1 error
    #################################################################
    def start(self):
    
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Starting the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
   
        # ------------------------------------------------------------
        # start R-GMA gin
        # ------------------------------------------------------------
        
        # check status of R-GMA gin
        status_output = commands.getstatusoutput("%s/etc/init.d/rgma-gin status" % os.environ['GLITE_LOCATION'])
        status = status_output[0]/256
        output = status_output[1]
            
        if status==0:
            
            # gin is running
            print 
            glib.printInfoMessage("%s is already running. Stopping and restarting %s ..." % (service.friendly_name, service.friendly_name))
            
            # stop gin first
            errorcode = os.system("%s/etc/init.d/rgma-gin stop" % os.environ['GLITE_LOCATION'])/256
            if errorcode:
                glib.printErrorMessage("Stopping %s failed." % service.friendly_name)
                return 1

        else:
            if status !=1:
                print       
                glib.printWarningMessage("%s is in unknown status. Trying to start %s ..." % (service.friendly_name, service.friendly_name))
                glib.printWarningMessage("Status script returned: %s" % output)

        print "Starting %s                                  " % service.friendly_name,

        # start gin            
        if os.path.exists("/var/lock/subsys/rgma-gin"):
            os.remove("/var/lock/subsys/rgma-gin")
            
        errorcode = os.system("%s/etc/init.d/rgma-gin start" % os.environ['GLITE_LOCATION'])/256
        if errorcode:
            glib.printFailedMessage()
            return 1

        glib.printOkMessage()
        return 0    
                        
            
    #################################################################
    # stop
    #
    # start gin
    #
    # return 0 ok
    #        1 error
    #################################################################
    def stop(self):
    
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Stopping the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
   
        # ------------------------------------------------------------
        # stop R-GMA gin
        # ------------------------------------------------------------
        
        # check status of R-GMA gin
        status_output = commands.getstatusoutput("%s/etc/init.d/rgma-gin status" % os.environ['GLITE_LOCATION'])
        status = status_output[0]/256
        output = status_output[1]
            
        if status==1:
            # gin is not running
            print
            glib.printInfoMessage("%s is not running." % service.friendly_name)
            return 0
        
        else:
            if status !=0:       
                print                
                glib.printWarningMessage("%s is in unknown status. Trying to stop %s ..." % (service.friendly_name, service.friendly_name))
                glib.printWarningMessage("Status script returned: %s" % output)

        print "Stopping %s                                  " % service.friendly_name,

        # stop gin            
        errorcode = os.system("%s/etc/init.d/rgma-gin stop" % os.environ['GLITE_LOCATION'])/256
        if errorcode:
            glib.printFailedMessage()
            return 1

        glib.printOkMessage()

        return 0    
            
        
    #################################################################
    # status
    #
    # print status of R-GMA gin
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def status(self):    
                
        # ------------------------------------------------------------
        # print status of rgma gin
        # ------------------------------------------------------------

        errorcode = os.system("%s/etc/init.d/rgma-gin status" % os.environ['GLITE_LOCATION'])
        
        if errorcode:
            print "\nThe %s is not running or not running correctly." % service.friendly_name
        else:
            print "\nThe %s is running." % service.friendly_name
            
        return errorcode            

    #################################################################
    # showConfiguration
    #
    # show configuration
    #
    # return 0         ok 
    #        1         error
    #################################################################
    def showConfiguration(self):
        
        self.copyright()
        self.showVersion()
        glib.print_params(params)

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "             Main settings of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        print "  R-GMA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.server.hostname']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.hostname' not found.")
        
        print "     --------------------------------------------------------------"
        print
        
        # print schema settings
        print "  R-GMA SCHEMA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.schema.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.schema.hostname']
        else:
            glib.printErrorMessage("Parameter 'rgma.schema.hostname' not found.")        
  
        print "     --------------------------------------------------------------"
        print
        
        # print registry settings
        print "  R-GMA REGISTRY SERVER"        
        print "     --------------------------------------------------------------"
# The following lines should be put in place as soon as multiple registries in enabled again
######
#        if params.has_key('rgma.registry.hostname'):
#            for item in params['rgma.registry.hostname']: 
#                print "     \033[34m[Location]\x1b[0m          %s " % item
#        else:
#            glib.printErrorMessage("Parameter 'rgma.registry.hostname' not found.")        
######

        # The following lines can be removed as soon as multiple registries are enabled again
        # start-new-block
        if params.has_key('rgma.registry.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.registry.hostname'][0]
        else:
            glib.printErrorMessage("Parameter 'rgma.registry.hostname' not found.")    
        
        if params.has_key('rgma.registry.hostname'):
            if len(params['rgma.registry.hostname']) > 1:
                print
                glib.printWarningMessage("You specified more than one registry.")
                glib.printWarningMessage("At the moment only one registry is allowed.")
                glib.printWarningMessage("Configuration will use first specified registry '%s'" % params['rgma.registry.hostname'][0])
                glib.printWarningMessage("and ignore the others.")
                print        
        # end-new-block

        print "     --------------------------------------------------------------"
        print

        print "  R-GMA GIN settings"        
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.gin.run_generic_info_provider'):
            print "     \033[34m[Run generic info provider]\x1b[0m    %s " % params['rgma.gin.run_generic_info_provider']
        else:
            glib.printErrorMessage("Parameter 'rgma.gin.run_generic_info_provider' not found.")        

        if params.has_key('rgma.gin.run_fmon_provider'):
            print "     \033[34m[Run FMON provider]\x1b[0m            %s " % params['rgma.gin.run_fmon_provider']
        else:
            glib.printErrorMessage("Parameter 'rgma.gin.run_fmon_provider' not found.")        

        if params.has_key('rgma.gin.run_ce_provider'):        
            print "     \033[34m[Run CE provider]\x1b[0m              %s " % params['rgma.gin.run_ce_provider']

        if params.has_key('rgma.secure.mode'):
            print "     \033[34m[SecureMode]\x1b[0m                   %s " % params['rgma.secure.mode']
        else:
            glib.printErrorMessage("Parameter 'rgma.secure.mode' not found.")  
         
        print "     --------------------------------------------------------------"
        print
                
        return 0
    

#################################################################
# loadDefaultParameters
#
# load the default parameters
#
# return 0 ok
#        1 error
#################################################################
def loadDefaultParameters(params):
   
    # rgma default params
    params['rgma.user.name']                    = "rgma"
    params['rgma.user.group']                   = "rgma"
    params['rgma.secure.mode']                  = "true"
    
    return 0


#################################################################
# set_env
#
# setup environment
# 
#################################################################
def set_env():

    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('CATALINA_HOME')
    glib.export('JAVA_HOME')

    # bin and lib paths
    glib.addEnvPath("PATH","%s/externals/bin:%s/bin:%s/bin" % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['JAVA_HOME']))
    glib.addEnvPath("LD_LIBRARY_PATH","%s/lib" % os.environ['GLITE_LOCATION'])

    # Set environment
    glib.setUserEnv()


#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':
        
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    verboseCLI = 0
    
    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------

    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Set default parameters
    loadDefaultParameters(params)
    
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'start', 'stop', 'status', 'version','siteconfig=','subservice','configure','cemon='])
        for o, a in opts:
            
            if o == "--verbose":
                verboseCLI = 1
                glib.verbose = 1
                
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
                
    except getopt.GetoptError:
        pass
    
    # load rgma common and rgma gin configuration parameters
    # ce is loade to get gin parameters - if not there: it is just ignored
    configFiles = ["%s/../glite-ce.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-rgma-gin.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-rgma-common.cfg.xml" % glib.getScriptPath()]
        
    if glib.loadConfiguration(configFiles,params):
        glib.printErrorMessage("Problems while loading configuration file.")
        sys.exit(1)
        
    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    verbose = 0
    if verboseCLI:
        verbose = 1
    else:
        if params.has_key('glite.installer.verbose'):
            if params['glite.installer.verbose'] == "true":
                verbose = 1
    
    glib.verbose = verbose
        
    # ------------------------------------------------------------    
    # Set up the environment
    # ------------------------------------------------------------    
    set_env()
          
    # ------------------------------------------------------------    
    # Instantiate the service classes
    # ------------------------------------------------------------    
    service = glite_rgma_gin()
    service.verbose = verbose
    
    # ------------------------------------------------------------
    # Command line opts if any
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'start', 'stop', 'status', 'version','siteconfig=','subservice','configure','cemon='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------
    subservice     = 0
    configure      = 0
    start          = 0
    cemon          = 0
    
    for o, a in opts:
        
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
            
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
            
        if o in ("-c", "--checkconf"):
            if service.showConfiguration():
                sys.exit(0)
            else:
                sys.exit(1)

        # run script as subservice
        if o == "--cemon":
            params['rgma.gin.run_ce_provider'] = a
            
        # run script as subservice
        if o == "--subservice":
            subservice = 1
            params['rgma.gin.run_as_subservice'] = "true"
            
        # configure service
        if o == "--configure":
            configure = 1
            
        # start service
        if o == "--start":
            start = 1            
        
        # stop service        
        if o == "--stop":
            if service.stop() == 0:
                sys.exit(0)
            else:
                glib.printErrorMessage("An error occurred while stopping the %s service" \
                    % service.friendly_name)
                sys.exit(1)
        
        # print status
        if o == "--status":
            if service.status() == 0:
                sys.exit(0)
            else:
                sys.exit(1)
                
    #-------------------------------------------------------------------------------    
    # Configure
    #-------------------------------------------------------------------------------    
    if configure==1:
        
        #-------------------------------------------------------------------------------    
        # Print configuration parameters
        #-------------------------------------------------------------------------------    
        if verbose:
            glib.print_params(params)

        #-------------------------------------------------------------------------------
        # print default statements
        #-------------------------------------------------------------------------------    
        if subservice==0:
            service.copyright()
            service.showVersion()
       
           #-------------------------------------------------------------------------------
        # stop gin first in case it is running and 
        # if script is not called as subservice
        #-------------------------------------------------------------------------------    
        if subservice==0:
            service.stop()

        # print banner
        service.banner()
         
        #-------------------------------------------------------------------------------
        # Configure the Service Publisher
        #-------------------------------------------------------------------------------    
        if service.configure() != 0:
            glib.printErrorMessage("\nAn unrecoverable error occurred while configuring the %s service" % service.friendly_name)
            sys.exit(1)
        else:
            print "\n%s configuration successfully completed\n" % service.friendly_name            
            if subservice==0:
                print "To start the %s please run" % service.friendly_name
                print "    %s/etc/config/scripts/glite-rgma-gin-config.py --start" % os.environ['GLITE_LOCATION']
            sys.exit(0)                            
    
    else:

        #-------------------------------------------------------------------------------    
        # Start
        #-------------------------------------------------------------------------------    
        if start==1:

            returncode = service.start()
            if  returncode == 0:
                print "\n%s successfully started\n" % service.friendly_name

                if subservice==0:
                    print "To check if the %s is running correctly you can run " % service.friendly_name
                    print "    %s/etc/config/scripts/glite-rgma-gin-config.py --status\n" % os.environ['GLITE_LOCATION']
                
                sys.exit(0)
            else:
                if returncode == -1:
                    # nothing to be done (not activated)
                    sys.exit(0)
                else:
                    glib.printErrorMessage("An error occurred while starting the %s service" % service.friendly_name)
                    sys.exit(1)

        else:
            # ------------------------------------------------------------
            # No option given
            # ------------------------------------------------------------
            service.usage()
            sys.exit(0)        
        
